VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "SavePDFClass"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private WithEvents oApplicationEvents As ApplicationEvents
Attribute oApplicationEvents.VB_VarHelpID = -1
 
Public Sub Monitor()
  Set oApplicationEvents = ThisApplication.ApplicationEvents
  MsgBox ("Save Monitoring turned on.")
End Sub

Private Sub oApplicationEvents_OnSaveDocument(ByVal DocumentObject As Document, ByVal BeforeOrAfter As EventTimingEnum, ByVal Context As NameValueMap, HandlingCode As HandlingCodeEnum)
  Call SavePDF
End Sub

Public Sub SavePDF()
'Check to see if this is an IDW, and that the file has a valid name.
'If not then skip the whole sub...
If ThisApplication.ActiveDocument.DocumentType = kDrawingDocumentObject And _
   ThisApplication.ActiveDocument.FullFileName <> "" Then
  
  Dim fs As Object
  Dim oDoc As Document
  Dim FlagOK As Boolean
  Dim ErrNum
  Dim oPrintMgr As DrawingPrintManager
    
  Set fs = CreateObject("Scripting.FileSystemObject")
  Set oDoc = ThisApplication.ActiveDocument
  
  'Look for an existing PDF file, and its availability...
  FlagOK = True
  If fs.FileExists(ChangeFileExtension(oDoc.FullFileName, ".pdf")) Then
    'We can only proceed if the existing pdf file is available, i.e. no-one
    'else is currently viewing it. We'll do a test re-name to determine it.
    On Error Resume Next
    Call fs.MoveFile(ChangeFileExtension(oDoc.FullFileName, ".pdf"), _
                     ChangeFileExtension(oDoc.FullFileName, "_1.pdf"))
    ErrNum = Err.Number
    On Error GoTo 0
    If ErrNum <> 0 Then
      MsgBox ("PDF Save Error. The existing PDF file is being accessed by someone else.")
      FlagOK = False
    End If
  End If
  
  If FlagOK Then
    Set oPrintMgr = oDoc.PrintManager
    'This is the name of the printer on my system. Probably should
    'Determine this by other means.
    oPrintMgr.Printer = "5D PDF Creator"
    oPrintMgr.NumberOfCopies = 1
    oPrintMgr.Orientation = kLandscapeOrientation
    oPrintMgr.ScaleMode = kPrintBestFitScale
    oPrintMgr.PaperSize = kPaperSizeA3
    oPrintMgr.PrintRange = kPrintAllSheets
    
    'Determine the filename, and create the postscript file. We must do this,
    'as printing to file is the only way to avoid getting a file dialog box
    'from the PDF Creator system. The printer is actually a postscript printer
    'connected to a postscript-to-PDF conversion port.
    oPrintMgr.PrintToFile (ChangeFileExtension(oDoc.FullFileName, ".eps"))
    
    'Wait for the EPS file to become available...
    'This part is messy. It is possible for the shell command below to start
    'executing, looking for the eps file, when the printer hasn't released the
    'file. These lines work reasonably well, but not always on slow machines.
    On Error Resume Next
    Do
      Open ChangeFileExtension(oDoc.FullFileName, ".eps") For Input As 1
      ErrNum = Err.Number
    Loop Until ErrNum = 0
    Close 1
    On Error GoTo 0
    
    'Convert the eps file to pdf.
    'ToPDF is a utility that comes with the PDF Creator software.
    'It converts the EPS to a PDF.
    'Note: It is not good practice to specify the path etc here.
    'We should probably go looking in the registry for the
    'correct location of this program. The location below
    'is for a default installation of the PDF system.
    Shell ("c:\program files\5d\5d pdf creator\topdf " + """" + _
          ChangeFileExtension(oDoc.FullFileName, ".eps") + """")
    
    'Delete the eps file.
    'We must loop here, as I don't know of a way to determine whether or not
    'the file is available. The EPS file is created in a different process
    'to Inventor, so we have to wait until that process has relased the file.
    On Error Resume Next
    Do While fs.FileExists(ChangeFileExtension(oDoc.FullFileName, ".eps"))
      fs.DeleteFile (ChangeFileExtension(oDoc.FullFileName, ".eps"))
    Loop
    On Error GoTo 0
  
    'Clean up the old file...
    If fs.FileExists(ChangeFileExtension(oDoc.FullFileName, ".pdf")) Then
      'The new PDF file exists. Delete the old one...
      If fs.FileExists(ChangeFileExtension(oDoc.FullFileName, "_1.pdf")) Then
        fs.DeleteFile (ChangeFileExtension(oDoc.FullFileName, "_1.pdf"))
      End If
    Else
      'The new PDF file doesn't exist. Replace the old one if it existed...
      MsgBox ("PDF Save Error. New PDF file not created.")
      If fs.FileExists(ChangeFileExtension(oDoc.FullFileName, "_1.pdf")) Then
        Call fs.MoveFile(ChangeFileExtension(oDoc.FullFileName, "_1.pdf"), _
                         ChangeFileExtension(oDoc.FullFileName, ".pdf"))
      End If
    End If
  End If 'FlagOK
End If 'document is a drawing file...
End Sub

Private Function ChangeFileExtension(FullFileName As String, Extension As String) As String
  'Find the rightmost decimal point...
  Dim i As Integer
  i = Len(FullFileName)
  Do While Mid(FullFileName, i, 1) <> "."
    i = i - 1
  Loop
  ChangeFileExtension = Left(FullFileName, i) & Extension
End Function

